package it.sagelab.dynaagen.autogen;

import nl.tno.dynaa.model.functional.Task;
import nl.tno.dynaa.model.functional.behavioural.*;
import nl.tno.dynaa.core.Event.EventType;

/*********************************************************************************
* Copyright (C) 2019 Simone Vuotto, UNISS <svuotto@uniss.it>
*
* Code autogenerated with DynaaGen <https://gitlab.sagelab.it/SimoV8/dynaagen>
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
* DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
* OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE
* OR OTHER DEALINGS IN THE SOFTWARE.
*
**********************************************************************************/

public class ClientTask {
    
    public static final EventType SEGMENT_BRANCH1 = Event.createType("SEGMENT_BRANCH1", "");
    public static final EventType SEGMENT_ELSE = Event.createType("SEGMENT_ELSE", "");
    
    public static Task build(int _pid, DataContainer globalVars) {

        final TransitionBehaviourSpecification behaviour = new TransitionBehaviourSpecification();
        final Task task = new Task(behaviour);
        task.setProperty("GLOBAL_VARS", globalVars);
        task.setProperty("_pid", _pid);
        
        final TaskSegment assertSegment0 = ClientTask.createAssertSegment0();
        final TaskSegment doSegment1 = ClientTask.createDoSegment1();
        final TaskSegment assingSegment2 = ClientTask.createAssingSegment2();
        final TaskSegment assingSegment3 = ClientTask.createAssingSegment3();
        final TaskSegment assertSegment4 = ClientTask.createAssertSegment4();
        final TaskSegment assingSegment5 = ClientTask.createAssingSegment5();
        final TaskSegment assingSegment6 = ClientTask.createAssingSegment6();
        
        behaviour.setInitialSegment(assertSegment0);
        behaviour.addTransition(assertSegment0, TaskSegment.SEGMENT_SUCCESS, doSegment1);
        behaviour.addTransition(doSegment1, TaskSegment.SEGMENT_SUCCESS, null);
        behaviour.addTransition(doSegment1, SEGMENT_BRANCH1, assingSegment2);
        behaviour.addTransition(doSegment1, SEGMENT_ELSE, assingSegment3);
        behaviour.addTransition(assingSegment2, TaskSegment.SEGMENT_SUCCESS, doSegment1);
        behaviour.addTransition(assingSegment3, TaskSegment.SEGMENT_SUCCESS, assertSegment4);
        behaviour.addTransition(assertSegment4, TaskSegment.SEGMENT_SUCCESS, assingSegment5);
        behaviour.addTransition(assingSegment5, TaskSegment.SEGMENT_SUCCESS, assingSegment6);
        behaviour.addTransition(assingSegment6, TaskSegment.SEGMENT_SUCCESS, doSegment1);
        
        return task;
    }

    private static TaskSegment createAssertSegment0(final Task task) {
        return new TaskSegment() {
            @Override
            public void run(final TaskContext taskContext) throws Exception {
                
                int _pid = task.get("_pid");
                assert(_pid>0);
                this.scheduleNow(TaskSegment.SEGMENT_SUCCESS);
            }
        };
    }
    
    private static TaskSegment createDoSegment1(final Task task) {
        return new TaskSegment() {
            @Override
            public void run(final TaskContext taskContext) throws Exception {
                
                ArrayList<Event.EventType> enabledEvents = new ArrayList<>();
                Random rand = new Random();
                
                int _pid = task.get("_pid");
                byte respond = task.get("GLOBAL_VARS").getRespond();
                if(respond!=_pid) {
                    enabledEvents.add(SEGMENT_BRANCH1);
                }
                enabledEvents.add(SEGMENT_BRANCH2);
                if(enabledEvents.isEmpty()) {
                    enabledEvents.add(SEGMENT_ELSE);
                }
                int _index = rand.nextInt(enabledEvents.size());
                this.scheduleNow(enabledEvents.get(_index));
            }
        };
    }
    
    private static TaskSegment createAssingSegment2(final Task task) {
        return new TaskSegment() {
            @Override
            public void run(final TaskContext taskContext) throws Exception {
                
                int _pid = task.get("_pid");
                byte request = task.get("GLOBAL_VARS").getRequest();
                task.get("GLOBAL_VARS").setRequest(_pid);
                this.scheduleNow(TaskSegment.SEGMENT_SUCCESS);
            }
        };
    }
    
    private static TaskSegment createAssingSegment3(final Task task) {
        return new TaskSegment() {
            @Override
            public void run(final TaskContext taskContext) throws Exception {
                
                byte cnt = task.get("GLOBAL_VARS").getCnt();
                task.get("GLOBAL_VARS").setCnt(cnt + 1);
                this.scheduleNow(TaskSegment.SEGMENT_SUCCESS);
            }
        };
    }
    
    private static TaskSegment createAssertSegment4(final Task task) {
        return new TaskSegment() {
            @Override
            public void run(final TaskContext taskContext) throws Exception {
                
                byte cnt = task.get("GLOBAL_VARS").getCnt();
                assert(cnt==1);
                this.scheduleNow(TaskSegment.SEGMENT_SUCCESS);
            }
        };
    }
    
    private static TaskSegment createAssingSegment5(final Task task) {
        return new TaskSegment() {
            @Override
            public void run(final TaskContext taskContext) throws Exception {
                
                byte cnt = task.get("GLOBAL_VARS").getCnt();
                task.get("GLOBAL_VARS").setCnt(cnt - 1);
                this.scheduleNow(TaskSegment.SEGMENT_SUCCESS);
            }
        };
    }
    
    private static TaskSegment createAssingSegment6(final Task task) {
        return new TaskSegment() {
            @Override
            public void run(final TaskContext taskContext) throws Exception {
                
                byte respond = task.get("GLOBAL_VARS").getRespond();
                task.get("GLOBAL_VARS").setRespond(0);
                this.scheduleNow(TaskSegment.SEGMENT_SUCCESS);
            }
        };
    }
    

}
