package it.sagelab.dynaagen.autogen;

import nl.tno.dynaa.model.functional.Task;
import nl.tno.dynaa.model.functional.behavioural.*;
import nl.tno.dynaa.core.Event.EventType;

/*********************************************************************************
* Copyright (C) 2019 Simone Vuotto, UNISS <svuotto@uniss.it>
*
* Code autogenerated with DynaaGen <https://gitlab.sagelab.it/SimoV8/dynaagen>
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
* EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
* DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
* OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE
* OR OTHER DEALINGS IN THE SOFTWARE.
*
**********************************************************************************/

public class ServerTask {
    
    public static final EventType SEGMENT_BRANCH1 = Event.createType("SEGMENT_BRANCH1", "");
    public static final EventType SEGMENT_BLOCK = Event.createType("SEGMENT_BLOCK", "");
    
    public static Task build(int _pid, DataContainer globalVars) {

        final TransitionBehaviourSpecification behaviour = new TransitionBehaviourSpecification();
        final Task task = new Task(behaviour);
        task.setProperty("GLOBAL_VARS", globalVars);
        task.setProperty("_pid", _pid);
        
        final TaskSegment doSegment0 = ServerTask.createDoSegment0();
        final TaskSegment assingSegment1 = ServerTask.createAssingSegment1();
        final TaskSegment exprSegment2 = ServerTask.createExprSegment2();
        final TaskSegment assingSegment3 = ServerTask.createAssingSegment3();
        
        behaviour.setInitialSegment(doSegment0);
        behaviour.addTransition(doSegment0, TaskSegment.SEGMENT_SUCCESS, null);
        behaviour.addTransition(doSegment0, SEGMENT_BRANCH1, assingSegment1);
        behaviour.addTransition(doSegment0, SEGMENT_BLOCK, doSegment0);
        behaviour.addTransition(assingSegment1, TaskSegment.SEGMENT_SUCCESS, exprSegment2);
        behaviour.addTransition(exprSegment2, TaskSegment.SEGMENT_SUCCESS, assingSegment3);
        behaviour.addTransition(exprSegment2, SEGMENT_BLOCK, exprSegment2);
        behaviour.addTransition(assingSegment3, TaskSegment.SEGMENT_SUCCESS, doSegment0);
        
        return task;
    }

    private static TaskSegment createDoSegment0(final Task task) {
        return new TaskSegment() {
            @Override
            public void run(final TaskContext taskContext) throws Exception {
                
                ArrayList<Event.EventType> enabledEvents = new ArrayList<>();
                Random rand = new Random();
                
                byte request = task.get("GLOBAL_VARS").getRequest();
                if(request!=0) {
                    enabledEvents.add(SEGMENT_BRANCH1);
                }
                if(enabledEvents.isEmpty()) {
                    enabledEvents.add(SEGMENT_BLOCK);
                }
                int _index = rand.nextInt(enabledEvents.size());
                this.scheduleNow(enabledEvents.get(_index));
            }
        };
    }
    
    private static TaskSegment createAssingSegment1(final Task task) {
        return new TaskSegment() {
            @Override
            public void run(final TaskContext taskContext) throws Exception {
                
                byte request = task.get("GLOBAL_VARS").getRequest();
                byte respond = task.get("GLOBAL_VARS").getRespond();
                task.get("GLOBAL_VARS").setRespond(request);
                this.scheduleNow(TaskSegment.SEGMENT_SUCCESS);
            }
        };
    }
    
    private static TaskSegment createExprSegment2(final Task task) {
        return new TaskSegment() {
            @Override
            public void run(final TaskContext taskContext) throws Exception {
                
                byte respond = task.get("GLOBAL_VARS").getRespond();
                if( respond==0) {
                    this.scheduleNow(TaskSegment.SEGMENT_SUCCESS);
                } else {
                    this.scheduleNow(SEGMENT_BLOCK);
                }
            }
        };
    }
    
    private static TaskSegment createAssingSegment3(final Task task) {
        return new TaskSegment() {
            @Override
            public void run(final TaskContext taskContext) throws Exception {
                
                byte request = task.get("GLOBAL_VARS").getRequest();
                task.get("GLOBAL_VARS").setRequest(0);
                this.scheduleNow(TaskSegment.SEGMENT_SUCCESS);
            }
        };
    }
    

}
