package it.sagelab.dynaagen.core;

import java.util.*;

public class DynaaSegment {
    public static String SEGMENT_SUCCESS = "TaskSegment.SEGMENT_SUCCESS";
    public static String SEGMENT_BLOCK = "SEGMENT_BLOCK";
    public static String SEGMENT_ELSE = "SEGMENT_ELSE";
    public static String SEGMENT_BRANCH = "SEGMENT_BRANCH";

    private final String name;
    private ArrayList<String> body;
    private final Map<String, DynaaSegment> nextSegments;

    public DynaaSegment(String name) {
        this.name = name;
        nextSegments = new HashMap<>();
        body = new ArrayList<>();
    }

    public void addLine(String line) {
        this.body.add(line);
    }

    public ArrayList<String> getBody() {
        return body;
    }

    public String getName() {
        return name;
    }

    public void addSegment(String event, DynaaSegment segment) {
        nextSegments.put(event, segment);
    }

    public void appendNext(DynaaSegment segment) {
        HashSet<DynaaSegment> visitedSegments = new HashSet<>();
        visitedSegments.add(this);
        appendNext(segment, visitedSegments);
    }

    private void appendNext(DynaaSegment segment, Set<DynaaSegment> visitedSegments) {
        if(nextSegments.containsKey(SEGMENT_SUCCESS) && nextSegments.get(SEGMENT_SUCCESS) == null)
            nextSegments.put(SEGMENT_SUCCESS, segment);
        else {
            for (DynaaSegment seg : nextSegments.values()) {
                if (!visitedSegments.contains(seg)) {
                    visitedSegments.add(seg);
                    seg.appendNext(segment, visitedSegments);
                }
            }
        }
    }

    public Map<String, DynaaSegment> getNextSegments() {
        return nextSegments;
    }
}
