package it.sagelab.dynaagen.core;

import it.sagelab.dynaagen.model.expr.*;

import java.util.HashSet;
import java.util.Set;

public class ExprToString {

    public static String parse(Expression expr) {
        StringBuilder builder = new StringBuilder();
        parse(expr, builder, true);
        return builder.toString();
    }

    private static void parse(Expression expr, StringBuilder builder, boolean outerExpr) {
        if(expr == null) {
            builder.append("null");
            return;
        }
        if(expr instanceof Variable)
            builder.append(((Variable) expr).getName());
        if(expr instanceof ConstantExpr)
            builder.append(((ConstantExpr) expr).getValue());
        if(expr instanceof UnaryOpExpr) {
            UnaryOpExpr unaryExpr = (UnaryOpExpr) expr;
            builder.append(((UnaryOpExpr) expr).getOp());
            parse(unaryExpr.getExpr(), builder, false);
        }
        if(expr instanceof BinaryOpExpr) {
            BinaryOpExpr binaryOpExpr = (BinaryOpExpr) expr;
            if(!outerExpr)
                builder.append("(");
            parse(binaryOpExpr.getLeftExpr(), builder, false);
            builder.append(binaryOpExpr.getOperator());
            parse(binaryOpExpr.getRightExpr(), builder, false);
            if(!outerExpr)
                builder.append(")");
        }
    }

    public static Set<Variable> findVariables(Expression expr) {
        HashSet<Variable> variables = new HashSet<>();
        findVariables(expr, variables);
        return variables;
    }

    private static void findVariables(Expression expr, HashSet<Variable> variables) {
        if(expr == null)
            return;
        if(expr instanceof Variable) {
            variables.add((Variable) expr);
        }
        if(expr instanceof UnaryOpExpr) {
            findVariables(((UnaryOpExpr) expr).getExpr(), variables);
        }
        if(expr instanceof BinaryOpExpr) {
            findVariables(((BinaryOpExpr) expr).getLeftExpr(), variables);
            findVariables(((BinaryOpExpr) expr).getRightExpr(), variables);
        }
    }

}
